/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */
package com.bonevich.eclipse.cvsgrapher;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import java.util.ResourceBundle;
import java.util.regex.Pattern;

/**
 * The main plugin class to be used in the desktop.
 */
public final class CvsGraphPlugin 
	extends AbstractUIPlugin 
	implements CvsGraphPluginResources
{
	public static final String PREFERENCE_COLOR_VERSION = "cvsgrapher.prefs.color.version";
	public static final String PREFERENCE_COLOR_BRANCH = "cvsgrapher.prefs.color.branch";
	public static final String PREFERENCE_MERGETAG_SOURCE = "cvsgrapher.prefs.mergetag.pattern.source";
	public static final String PREFERENCE_MERGETAG_DEST = "cvsgrapher.prefs.mergetag.pattern.dest";
	public static final String PREFERENCE_SHOW_AUTHOR = "cvsgrapher.prefs.show.author";
	public static final String PREFERENCE_SHOW_TIMESTAMP = "cvsgrapher.prefs.show.timestamp";
	public static final String PREFERENCE_SHOW_TAGS = "cvsgrapher.prefs.show.tags";
	
	public static final String DEFAULT_COLOR_VERSION = "211,211,211"; //lightGray
	public static final String DEFAULT_COLOR_BRANCH = "255,255,0"; //yellow

	public static final String DEFAULT_MERGETAG_SOURCE = "source_(.*)";
	public static final String DEFAULT_MERGETAG_DEST = "merge_(.*)";
	
	public static final boolean DEFAULT_SHOW_AUTHOR = true;
	public static final boolean DEFAULT_SHOW_TIMESTAMP = true;
	public static final boolean DEFAULT_SHOW_TAGS = true;
	
	//The shared instance.
	private static CvsGraphPlugin _currentInstance;
	private ColorConverter _converter;
	private Pattern _sourcePattern;
	private Pattern _destPattern;
	
	public CvsGraphPlugin(IPluginDescriptor descriptor)
	{
		super(descriptor);
		_currentInstance = this;
		_converter = new ColorConverter();
	}

	public static CvsGraphPlugin getPlugin()
	{
		return _currentInstance;
	}

	public static IWorkspace getWorkspace()
	{
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key)
	{
		return CvsGraphPluginMessages.bind(key);
	}

	public ResourceBundle getResourceBundle()
	{
		return CvsGraphPluginMessages.getResourceBundle();
	}

	protected void initializeDefaultPreferences(IPreferenceStore store)
	{
		store.setDefault(PREFERENCE_COLOR_VERSION, DEFAULT_COLOR_VERSION);
		store.setDefault(PREFERENCE_COLOR_BRANCH, DEFAULT_COLOR_BRANCH);
		store.setDefault(PREFERENCE_MERGETAG_SOURCE, DEFAULT_MERGETAG_SOURCE);
		store.setDefault(PREFERENCE_MERGETAG_DEST, DEFAULT_MERGETAG_DEST);
		store.setDefault(PREFERENCE_SHOW_AUTHOR, DEFAULT_SHOW_AUTHOR);
		store.setDefault(PREFERENCE_SHOW_TIMESTAMP, DEFAULT_SHOW_TIMESTAMP);
		store.setDefault(PREFERENCE_SHOW_TAGS, DEFAULT_SHOW_TAGS);
	}
	
	public Color getVersionNodeColorPreference()
	{
		return _converter.getColor(getPreferenceStore().getString(PREFERENCE_COLOR_VERSION));
	}

	public Color getBranchNodeColorPreference()
	{
		return _converter.getColor(getPreferenceStore().getString(PREFERENCE_COLOR_BRANCH));
	}

	public Pattern getMergeTagSourcePreference()
	{
		String patternString = getPreferenceStore().getString(PREFERENCE_MERGETAG_SOURCE);
		if (_sourcePattern == null || !_sourcePattern.pattern().equals(patternString))
		{
			_sourcePattern = Pattern.compile(patternString, Pattern.CASE_INSENSITIVE);
		}
		return _sourcePattern;
	}

	public Pattern getMergeTagDestPreference()
	{
		String patternString = getPreferenceStore().getString(PREFERENCE_MERGETAG_DEST);
		if (_destPattern == null || !_destPattern.pattern().equals(patternString))
		{
			_destPattern = Pattern.compile(patternString, Pattern.CASE_INSENSITIVE);
		}
		return _destPattern;
	}
	
	public boolean showAuthorPreference()
	{
		return getPreferenceStore().getBoolean(PREFERENCE_SHOW_AUTHOR);
	}
	
	public boolean showTimestampPreference()
	{
		return getPreferenceStore().getBoolean(PREFERENCE_SHOW_TIMESTAMP);
	}
	
	public boolean showTagsPreference()
	{
		return getPreferenceStore().getBoolean(PREFERENCE_SHOW_TAGS);
	}
	
}