/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */
package com.bonevich.eclipse.cvsgrapher.editors;

import com.bonevich.eclipse.cvsgrapher.model.ICvsGraphNodeModel;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;

/**
 * Base class for node (version, branch) controller edit parts.
 * 
 * @author jbonevic
 */
public abstract class AbstractCvsGraphNodeEditPart
	extends AbstractGraphicalEditPart
	implements ICvsGraphNodeEditPart, PropertyChangeListener
{
	protected static final Font VERSION   = new Font(null, "Helvetica", 10, SWT.BOLD);
	protected static final Font INFO = new Font(null, "Helvetica", 8, SWT.NORMAL);
	
	/** The figure's anchor. */
	private ConnectionAnchor _anchor;

	/**
	 * Solely to intercept events in the graph. This way the model objects are
	 * not sullied with view information (location, size).
	 */
	private CvsGraphNodeViewHelper _helper;

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies()
	{
		// For some reasons I do not quite understand, the
		// CvsGraphXYLayoutEditPolicy.createChildEditPolicy() must be 
		// used instead of a PRIMARY_DRAG_ROLE edit policy here.
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals()
	{
//		System.out.println("[AbstractCvsGraphNodeEditPart] refreshVisuals");
		Point loc = _helper.getLocation();
//		System.out.println("[AbstractCvsGraphNodeEditPart] refreshVisuals: location = " + loc);
		if (loc != null)
		{
			Rectangle r = new Rectangle(loc, CvsGraphNodeViewHelper.PREFERRED_SIZE);
			((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), r);
		}
	}

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt)
	{
		String prop = evt.getPropertyName();
		if (prop.equals("size") || prop.equals("location"))
		{
			refreshVisuals();
		}
	}

	/**
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate()
	{
		if (isActive() == false)
		{
			super.activate();
			_helper.addPropertyChangeListener(this);
		}
	}

	/**
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate()
	{
		if (isActive())
		{
			super.deactivate();
			_helper.removePropertyChangeListener(this);
		}
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections()
	{
		return getNodeModel().getSourceConnections();
	}
	
	protected ICvsGraphNodeModel getNodeModel()
	{
		return ((ICvsGraphNodeModel) getModel());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections()
	{
		return getNodeModel().getTargetConnections();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createConnection(Object)
	 */
	protected ConnectionEditPart createConnection(Object iModel)
	{
		// The line below is based on Logical Diagram Editor's LogicEditPart.createConnection()
		// It looks like an EditPart cache mechanism for EditPart connections, but I need
		// to make sure of that...
		CvsGraphConnectionEditPart connectPart =
			(CvsGraphConnectionEditPart) getRoot().getViewer().getEditPartRegistry().get(iModel);
		if (connectPart == null)
		{
			connectPart = new CvsGraphConnectionEditPart();
			connectPart.setModel(iModel);
		}

		return connectPart;
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(ConnectionEditPart)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection)
	{
		return _anchor;
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request)
	{
		return null;
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
	{
		return _anchor;
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request)
	{
		return null;
	}

	protected void setViewHelper(CvsGraphNodeViewHelper helper)
	{
		_helper = helper;
	}

	public CvsGraphNodeViewHelper getViewHelper()
	{
		return _helper;
	}

	protected void setAnchor(ConnectionAnchor anchor)
	{
		_anchor = anchor;
	}

}
