/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */
package com.bonevich.eclipse.cvsgrapher.editors;

import com.bonevich.eclipse.cvsgrapher.model.CvsGraphVersionModel;

import org.eclipse.compare.CompareUI;
import org.eclipse.gef.*;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.team.internal.ccvs.core.ILogEntry;
import org.eclipse.team.internal.ccvs.ui.CVSCompareEditorInput;
import org.eclipse.team.internal.ccvs.ui.ResourceEditionNode;

import java.util.List;

/**
 * The popup context menu for the CVS Graph editor.
 * 
 * @author jbonevic
 */
public final class CvsGraphContextMenu extends ContextMenuProvider
{
	private ZoomManager _zoomManager;
	
	public CvsGraphContextMenu(GraphicalViewer viewer, ZoomManager manager)
	{
		super(viewer);
		_zoomManager = manager;
	}

	public void buildContextMenu(IMenuManager menu)
	{
		menu.add(new ResetAction());
		menu.add(new CompareAction());
		
		menu.add(new Separator());

		menu.add(new GraphZoomInAction());
		menu.add(new GraphZoomOutAction());
	}

	private final class GraphZoomInAction extends ZoomInAction
	{
		public GraphZoomInAction()
		{
			super(_zoomManager);
		}
		
		public boolean isEnabled()
		{
			return _zoomManager.canZoomIn();
		}
	}
	
	private final class GraphZoomOutAction extends ZoomOutAction
	{
		public GraphZoomOutAction()
		{
			super(_zoomManager);
		}
		
		public boolean isEnabled()
		{
			return _zoomManager.canZoomOut();
		}
	}
	
	private final class ResetAction extends Action
	{
		public String getText()
		{
			return "Reset Graph";
		}

		public String getToolTipText()
		{
			return "Reset the version graph to its original layout";
		}

		public void run()
		{
			GraphicalEditPart part = (GraphicalEditPart) getViewer().getContents();
			CvsGraphLayout layout = (CvsGraphLayout) part.getFigure().getLayoutManager();
			layout.reset();
			_zoomManager.setZoom(1.0d);
		}
	}

	private final class CompareAction extends Action
	{
		public String getText()
		{
			return "Compare Versions";
		}

		public String getToolTipText()
		{
			return "Compare the two selected versions";
		}

		public void run()
		{
			List parts = getViewer().getSelectedEditParts();
			EditPart part1 = (EditPart) parts.get(0);
			EditPart part2 = (EditPart) parts.get(1);
			
			CvsGraphVersionModel version1 = (CvsGraphVersionModel) part1.getModel();
			CvsGraphVersionModel version2 = (CvsGraphVersionModel) part2.getModel();
			
			ILogEntry entry1 = version1.getEntry();
			ILogEntry entry2 = version2.getEntry();
			
			ResourceEditionNode left = new ResourceEditionNode(entry1.getRemoteFile());
			ResourceEditionNode right = new ResourceEditionNode(entry2.getRemoteFile());
			CompareUI.openCompareEditor(
				new CVSCompareEditorInput(left, right)
			);
		}

		public boolean isEnabled()
		{
			List parts = getViewer().getSelectedEditParts();
			if (parts.size() == 2)
			{
				EditPart part1 = (EditPart) parts.get(0);
				EditPart part2 = (EditPart) parts.get(1);
				if (part1 instanceof CvsGraphVersionEditPart && part2 instanceof CvsGraphVersionEditPart)
				{
					return true;
				}
			}
			return false;
		}

	}
	
}
