/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */
package com.bonevich.eclipse.cvsgrapher.prefs;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import com.bonevich.eclipse.cvsgrapher.CvsGraphPlugin;
import com.bonevich.eclipse.cvsgrapher.CvsGraphPluginMessages;
import com.bonevich.eclipse.cvsgrapher.CvsGraphPluginResources;

/**
 * Preference page for the Boneclipse Logging Plugin.  Allows the user
 * to specify which logging API to use and the default name for the logging
 * field to be used.
 * 
 * @author jbonevic
 */
public class CvsGrapherPreferencePage 
	extends PreferencePage 
	implements IWorkbenchPreferencePage, CvsGraphPluginResources
{
	private static final String PREFERENCE_PAGE_TITLE    = "CVSGrapher";

	// field editor keys
	private static final String FIELDEDITOR_COLOR_VERSION = "cvsgrapher.color.version";
	private static final String FIELDEDITOR_COLOR_BRANCH = "cvsgrapher.color.branch";
	
	private static final String FIELDEDITOR_MERGETAG_SOURCE = "cvsgrapher.mergetag.pattern.source";
	private static final String FIELDEDITOR_MERGETAG_DEST = "cvsgrapher.mergetag.pattern.destination";
	
	private static final String FIELDEDITOR_SHOW_AUTHOR = "cvsgrapher.show.author";
	private static final String FIELDEDITOR_SHOW_TIMESTAMP = "cvsgrapher.show.timestamp";
	private static final String FIELDEDITOR_SHOW_TAGS = "cvsgrapher.show.tags";
	
	private Map _fieldEditors = new HashMap();
	
	public CvsGrapherPreferencePage()
	{
		super(PREFERENCE_PAGE_TITLE);
	}

	public void init(IWorkbench workbench)
	{
		setPreferenceStore(CvsGraphPlugin.getPlugin().getPreferenceStore());
	}

	protected Control createContents(Composite parent)
	{
		Composite top = createContainerWithDescription(parent);

		createColorFieldEditors(top);
		createMergeTagPatternFields(top);
		createShowAttributesCheckboxGroup(top);
		
		return top;
	}

	private void createColorFieldEditors(Composite top)
	{
		Composite colorFieldCmp = new Composite(top, SWT.LEFT);
		colorFieldCmp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		colorFieldCmp.setLayout(new GridLayout());
		
		ColorFieldEditor versionColorField = new ColorFieldEditor(
				CvsGraphPlugin.PREFERENCE_COLOR_VERSION,
				CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_COLOR_VERSION_LABEL),
				colorFieldCmp 
		);

		ColorFieldEditor branchColorField = new ColorFieldEditor(
				CvsGraphPlugin.PREFERENCE_COLOR_BRANCH,
				CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_COLOR_BRANCH_LABEL),
				colorFieldCmp 
		);
		addFieldEditor(FIELDEDITOR_COLOR_BRANCH, branchColorField);
		addFieldEditor(FIELDEDITOR_COLOR_VERSION, versionColorField);
	}

	private void createShowAttributesCheckboxGroup(Composite top)
	{
		Group group1 = new Group(top, SWT.LEFT);
		group1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout layout = new GridLayout();
		layout.marginWidth = 7;
		group1.setLayout(layout);
		group1.setText(CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_SHOW_LABEL));

		Composite group = new Composite(group1, SWT.NONE);
		group.setLayoutData(new GridData(GridData.CENTER));
		group.setLayout(new GridLayout());
		
		BooleanFieldEditor useTraceConditionals = new BooleanFieldEditor(
				CvsGraphPlugin.PREFERENCE_SHOW_AUTHOR,
				CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_SHOW_AUTHOR_LABEL),
				group
		);
		BooleanFieldEditor useDebugConditionals = new BooleanFieldEditor(
				CvsGraphPlugin.PREFERENCE_SHOW_TIMESTAMP,
				CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_SHOW_TIMESTAMP_LABEL),
				group
		);
		BooleanFieldEditor useInfoConditionals = new BooleanFieldEditor(
				CvsGraphPlugin.PREFERENCE_SHOW_TAGS,
				CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_SHOW_TAGS_LABEL),
				group
		);

		addFieldEditor(FIELDEDITOR_SHOW_AUTHOR, useTraceConditionals);
		addFieldEditor(FIELDEDITOR_SHOW_TIMESTAMP, useDebugConditionals);
		addFieldEditor(FIELDEDITOR_SHOW_TAGS, useInfoConditionals);
	}

	private void createMergeTagPatternFields(Composite top)
	{
		Group group1 = new Group(top, SWT.LEFT);
		group1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout layout = new GridLayout();
		layout.marginWidth = 7;
		group1.setLayout(layout);
		group1.setText(CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_MERGETAGS_LABEL));

		Composite mergeTagPatternComp = new Composite(group1, SWT.LEFT);
		mergeTagPatternComp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		mergeTagPatternComp.setLayout(new GridLayout());
		
		StringFieldEditor sourceMergeTagPattern = new StringFieldEditor(
				CvsGraphPlugin.PREFERENCE_MERGETAG_SOURCE,
				CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_MERGETAG_SOURCE_LABEL),
				15,
				StringFieldEditor.VALIDATE_ON_FOCUS_LOST,
				mergeTagPatternComp
			);
		sourceMergeTagPattern.setEmptyStringAllowed(true);

		StringFieldEditor destMergeTagPattern = new StringFieldEditor(
				CvsGraphPlugin.PREFERENCE_MERGETAG_DEST,
				CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_MERGETAG_DEST_LABEL),
				15,
				StringFieldEditor.VALIDATE_ON_FOCUS_LOST,
				mergeTagPatternComp
		);
		destMergeTagPattern.setEmptyStringAllowed(true);

		addFieldEditor(FIELDEDITOR_MERGETAG_SOURCE, sourceMergeTagPattern);
		addFieldEditor(FIELDEDITOR_MERGETAG_DEST, destMergeTagPattern);
	}

	private Composite createContainerWithDescription(Composite parent)
	{
		Composite top = new Composite(parent, SWT.LEFT);
		top.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		top.setLayout(new GridLayout());
		
		Label description = new Label(top, SWT.NONE);
		description.setText(CvsGraphPluginMessages.bind(CVSGRAPHER_PREFS_GENERAL));
		return top;
	}

	private void addFieldEditor(String name, FieldEditor editor)
	{
		_fieldEditors.put(name, editor);
		editor.setPreferencePage(this);
		editor.setPreferenceStore(getPreferenceStore());
		editor.load();
	}
	
	private FieldEditor getFieldEditor(String name)
	{
		return (FieldEditor) _fieldEditors.get(name);
	}

	protected void performDefaults()
	{
		super.performDefaults();

		loadDefault(FIELDEDITOR_COLOR_VERSION);
		loadDefault(FIELDEDITOR_COLOR_BRANCH);
		loadDefault(FIELDEDITOR_MERGETAG_SOURCE);
		loadDefault(FIELDEDITOR_MERGETAG_DEST);
		loadDefault(FIELDEDITOR_SHOW_AUTHOR);
		loadDefault(FIELDEDITOR_SHOW_TIMESTAMP);
		loadDefault(FIELDEDITOR_SHOW_TAGS);
	}
	
	private void loadDefault(String field)
	{
		FieldEditor editor = getFieldEditor(field);
		editor.loadDefault();
	}

	public boolean performOk()
	{
		storeValue(FIELDEDITOR_COLOR_VERSION);
		storeValue(FIELDEDITOR_COLOR_BRANCH);
		storeValue(FIELDEDITOR_MERGETAG_SOURCE);
		storeValue(FIELDEDITOR_MERGETAG_DEST);
		storeValue(FIELDEDITOR_SHOW_AUTHOR);
		storeValue(FIELDEDITOR_SHOW_TIMESTAMP);
		storeValue(FIELDEDITOR_SHOW_TAGS);
		
		return super.performOk();
	}
	
	private void storeValue(String field)
	{
		FieldEditor editor = getFieldEditor(field);
		editor.store();
	}

}
